@@ -16,6 +16,8 @@
  * along with Conboy. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "localisation.h"
+
 #include <gtk/gtk.h>
 #include <tablet-browser-interface.h>
 #include <sys/socket.h>
@@ -33,32 +35,30 @@
 static gint
 web_sync_send_notes(GList *notes, gchar *url, gint expected_rev, time_t last_sync_time, gint *uploaded_notes, GError **error)
 {
-	gchar *t_key = settings_load_oauth_access_token();
-	gchar *t_secret = settings_load_oauth_access_secret();
-
 	/*
 	 * Create correct json structure to send the note
 	 */
-	JsonNode *result = json_node_new(JSON_NODE_OBJECT);
-	JsonObject *obj = json_object_new();
 	JsonArray *array = json_array_new();
 
-	while (notes) {
-		ConboyNote *note = CONBOY_NOTE(notes->data);
+	GList *iter = notes;
+	while (iter) {
+		ConboyNote *note = CONBOY_NOTE(iter->data);
 		if (note->last_metadata_change_date > last_sync_time) {
 			g_printerr("Will send: %s\n", note->title);
 			JsonNode *note_node = json_get_node_from_note(note);
 			json_array_add_element(array, note_node);
 		}
-		notes = notes->next;
+		iter = iter->next;
 	}
 
 	*uploaded_notes = json_array_get_length(array);
 	if (*uploaded_notes == 0) {
 		g_printerr("INFO: No new notes on client. Sending nothing.\n");
+		json_array_unref(array);
 		return expected_rev - 1;
 	}
 
+	JsonObject *obj = json_object_new();
 	JsonNode *node = json_node_new(JSON_NODE_ARRAY);
 	json_node_set_array(node, array);
 	json_object_add_member(obj, "note-changes", node);
@@ -67,18 +67,22 @@ web_sync_send_notes(GList *notes, gchar *url, gint expected_rev, time_t last_syn
 	json_node_set_int(node, expected_rev);
 	json_object_add_member(obj, "latest-sync-revision", node);
 
+	JsonNode *result = json_node_new(JSON_NODE_OBJECT);
 	json_node_take_object(result, obj);
 
 	/* Convert to string */
 	gchar *json_string = json_node_to_string(result, FALSE);
 
+	/* Freeing the root node should also free the children */
+	json_node_free(result);
+
 	g_printerr("&&&&&&&&&&&&&&&&&&\n");
 	g_printerr("%s\n", json_string);
 	g_printerr("&&&&&&&&&&&&&&&&&&\n");
 
 
 	gchar *reply = conboy_http_put(url, json_string, TRUE);
-
+	g_free(json_string);
 	/*
 	g_printerr("Reply from Snowy:\n");
 	g_printerr("%s\n", reply);
@@ -86,6 +90,8 @@ web_sync_send_notes(GList *notes, gchar *url, gint expected_rev, time_t last_syn
 
 	/* Parse answer and see if expected_rev fits or not */
 	JsonNoteList *note_list = json_get_note_list(reply);
+	/* TODO: JsonNoteList plus containing Notes needs to be freed or unrefed */
+	g_free(reply);
 
 	if (note_list == NULL) {
 		g_set_error(error, 0, 0, "json_get_note_list() returned NULL");
@@ -102,13 +108,13 @@ web_sync_send_notes(GList *notes, gchar *url, gint expected_rev, time_t last_syn
 }
 
 static JsonNoteList*
-web_sync_get_notes(JsonUser *user, int since_rev)
+web_sync_get_notes(JsonUser *user, int since_rev, gboolean with_content)
 {
 	JsonNoteList *result;
 	gchar *json_string;
 	gchar get_all_notes_url[1024];
 
-	g_sprintf(get_all_notes_url, "%s?include_notes=true&since=%i", user->api_ref, since_rev);
+	g_sprintf(get_all_notes_url, "%s?include_notes=%s&since=%i", user->api_ref, with_content ? "true" : "false", since_rev);
 
 	json_string = conboy_http_get(get_all_notes_url, TRUE);
 	result = json_get_note_list(json_string);
@@ -138,6 +144,9 @@ web_sync_pulse_bar (GtkProgressBar *bar)
 }
 
 
+/**
+ * Removes a note from a list of notes
+ */
 static GList*
 web_sync_remove_by_guid(GList *list, ConboyNote *note_to_remove)
 {
@@ -202,11 +211,10 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 	gint changed_notes = 0;
 
 	/* Create list of all local notes */
-	ConboyNoteStore *note_store = app_data->note_store;
-	GList *local_notes = conboy_note_store_get_all(note_store);
+	GList *local_notes = conboy_note_store_get_all(CONBOY_NOTE_STORE(app_data->note_store));
 
 	/* Get all notes since last syncRev*/
-	JsonNoteList *server_note_list = web_sync_get_notes(user, *last_sync_rev);
+	JsonNoteList *server_note_list = web_sync_get_notes(user, *last_sync_rev, TRUE);
 	*last_sync_rev = server_note_list->latest_sync_revision;
 
 	/* Save notes */
@@ -221,15 +229,19 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 		/* If not yet in the note store, add this note */
 		if (!conboy_note_store_find_by_guid(app_data->note_store, server_note->guid)) {
 			g_printerr("INFO: Adding note '%s' to note store\n", server_note->title);
+			gdk_threads_enter();
 			conboy_note_store_add(app_data->note_store, server_note, NULL);
+			gdk_threads_leave();
 			added_notes++;
 		} else {
 			g_printerr("INFO: Updating note '%s' in note store\n", server_note->title);
 			ConboyNote *local_note = conboy_note_store_find_by_guid(app_data->note_store, server_note->guid);
 
 			/* Replace local note with the updated version from server */
+			gdk_threads_enter();
 			conboy_note_store_remove(app_data->note_store, local_note);
 			conboy_note_store_add(app_data->note_store, server_note, NULL);
+			gdk_threads_leave();
 			changed_notes++;
 
 			/* Compare note timestamp with last_sync_time. If it's bigger, the note has been also modified locally. */
@@ -237,7 +249,7 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 				g_printerr("INFO: We have a conflict\n");
 
 				/* TODO: Prompt user, ask for overwrite or not. If not overwrite, ask for new name */
-				gboolean overwrite = FALSE;
+				gboolean overwrite = TRUE;
 
 				if (!overwrite) { /* If we do not overwrite, we need to create a copy that note */
 					ConboyNote *rescue_note = conboy_note_copy(local_note);
@@ -251,7 +263,9 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 					/* Save rescue_note */
 					conboy_storage_note_save(app_data->storage, rescue_note);
 					/* Add to note store */
+					gdk_threads_enter();
 					conboy_note_store_add(app_data->note_store, rescue_note, NULL);
+					gdk_threads_leave();
 					/* Add to temp list of local notes */
 					local_notes = g_list_append(local_notes, rescue_note);
 
@@ -259,9 +273,9 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 				}
 			}
 		}
+		/* TODO: JsonNoteList *server_note_list needs to be freed or unrefed */
 
 		/* Remove from list of local notes */
-		/* Find local note and remove from list */
 		local_notes = web_sync_remove_by_guid(local_notes, server_note);
 
 		server_notes = server_notes->next;
@@ -273,7 +287,231 @@ web_sync_incoming_changes(JsonUser *user, gint *last_sync_rev, time_t last_sync_
 	return local_notes;
 }
 
+/**
+ * TODO: This implementation is very naiv. It opens and reads through the file each time it is
+ * called. This should probably be cached somewhere...
+ */
+static gboolean
+note_has_been_synced_at_least_once(const gchar *guid)
+{
+	g_return_val_if_fail(guid != NULL, FALSE);
+	g_return_val_if_fail(strcmp("", guid) != 0, FALSE);
+
+	gchar *filename = g_strconcat(g_get_home_dir(), "/.conboy/synced_notes.txt", NULL);
+	if (!g_file_test(filename, G_FILE_TEST_EXISTS)) {
+		g_free(filename);
+		return FALSE;
+	}
+
+	gboolean result = FALSE;
+	GIOChannel *channel = g_io_channel_new_file(filename, "r", NULL);
+	gchar *line = NULL;
+	while (g_io_channel_read_line(channel, &line, NULL, NULL, NULL) != G_IO_STATUS_EOF) {
+		if (line != NULL) {
+			if (strncmp(guid, line, 36) == 0) {
+				result = TRUE;
+			}
+			g_free(line);
+		}
+		if (result) break;
+	}
+
+	g_io_channel_shutdown(channel, TRUE, NULL);
+	g_io_channel_unref(channel);
+	g_free(filename);
+
+	return result;
+}
+
+static void
+update_synced_notes_file()
+{
+	AppData *app_data = app_data_get();
+	gchar *filename = g_strconcat(g_get_home_dir(), "/.conboy/synced_notes.txt", NULL);
+
+	/* Create the content of the file */
+	GString *content = g_string_new("");
+	GList *all_local_notes = conboy_note_store_get_all(app_data->note_store);
+	GList *iter = all_local_notes;
+	while (iter) {
+		ConboyNote *note = CONBOY_NOTE(iter->data);
+		g_string_append(content, note->guid);
+		g_string_append(content, "\n");
+		iter = iter->next;
+	}
+
+	/* Write the file */
+	if (!g_file_set_contents(filename, content->str, -1, NULL)) {
+		g_printerr("ERROR: Could not write to file: %s\n", filename);
+	}
+
+	/* Free stuff */
+	g_list_free(all_local_notes);
+	g_string_free(content, TRUE);
+	g_free(filename);
+}
+
+static GList*
+web_sync_delete_local_notes(JsonUser *user, GList *notes_to_upload, gint *deleted_notes_count)
+{
+	/*
+	 * 1. Get since rev=0 without content
+	 * 2. Compare to local notes
+	 * 3. Delete local difference
+	 */
+	AppData *app_data = app_data_get();
+
+	JsonNoteList *json_server_notes = web_sync_get_notes(user, 0, FALSE);
+	GSList *server_notes = json_server_notes->notes;
+	GList *local_notes = conboy_note_store_get_all(CONBOY_NOTE_STORE(app_data->note_store));
+
+	/* All that is not on the server has to be deleted locally */
+	/* Go though list of local notes and everything that cannot be found in the
+	 * list of server notes, must be deleted locally.
+	 */
+
+	GList *local_notes_iter = local_notes;
+	while (local_notes_iter != NULL) {
+
+		ConboyNote *local_note = CONBOY_NOTE(local_notes_iter->data);
+
+		gboolean found_note = FALSE;
+		GSList *server_notes_iter = server_notes;
+		while (server_notes_iter != NULL) {
+
+			ConboyNote *server_note = CONBOY_NOTE(server_notes_iter->data);
+
+			if (strcmp(local_note->guid, server_note->guid) == 0) {
+				found_note = TRUE;
+				break;
+			}
+
+			server_notes_iter = server_notes_iter->next;
+		}
+
+		/* If we couldn't find the local note on the server - delete it but only if it has been synced before */
+		if (!found_note) {
+			if (note_has_been_synced_at_least_once(local_note->guid)) {
+				g_printerr("Deleting note: %s\n", local_note->title);
+				/* Delete from filesystem etc. */
+				gdk_threads_enter();
+				note_delete(local_note);
+				gdk_threads_leave();
+				/* Remove from notes_to_update if on it */
+				notes_to_upload = web_sync_remove_by_guid(notes_to_upload, local_note);
+				(*deleted_notes_count)++;
+			} else {
+				g_printerr("INFO: Not deleting '%s', because it was never synced before\n", local_note->title);
+			}
+		}
+
+		local_notes_iter = local_notes_iter->next;
+	}
+
+	g_list_free(local_notes);
+	/* TODO: JsonNoteList *json_server_notes needs to be freed or unrefed */
+	/* We need to make sure that memory is given back, but ConboyNotes that are now local notes are not deleted */
+	//json_server_notes_free(json_server_notes);
+
+	return notes_to_upload;
+}
+
+
+static gint
+web_sync_send_local_deletions(gchar *url, gint expected_rev, gint *local_deletions, GError **error)
+{
+	gchar *content = NULL;
+	gchar *filename = g_strconcat(g_get_home_dir(), "/.conboy/deleted_notes.txt", NULL);
+	g_file_get_contents(filename, &content, NULL, NULL);
+	if (content == NULL) {
+		g_printerr("INFO: File empty or does not exist. Nothing to delete\n", filename);
+		g_free(filename);
+		return expected_rev - 1;
+	}
+	g_free(filename);
+
+	JsonArray *array = json_array_new();
+	gchar **parts = g_strsplit(content, "\n", -1);
+	int i = 0;
+	while (parts[i] != NULL) {
+		gchar *guid = parts[i];
+		if (strcmp(guid, "") != 0) {
+			if (note_has_been_synced_at_least_once(guid)) {
+				JsonNode *note_node = json_get_delete_node(guid);
+				json_array_add_element(array, note_node);
+			}
+		}
+		i++;
+	}
+	g_strfreev(parts);
+
+	*local_deletions = json_array_get_length(array);
+	if (*local_deletions == 0) {
+		g_printerr("INFO: No notes deleted on client. Sending nothing.\n");
+		json_array_unref(array);
+		return expected_rev - 1;
+	}
+
+	JsonObject *obj = json_object_new();
+	JsonNode *node = json_node_new(JSON_NODE_ARRAY);
+	json_node_set_array(node, array);
+	json_object_add_member(obj, "note-changes", node);
+
+	node = json_node_new(JSON_NODE_VALUE);
+	json_node_set_int(node, expected_rev);
+	json_object_add_member(obj, "latest-sync-revision", node);
+
+	JsonNode *result = json_node_new(JSON_NODE_OBJECT);
+	json_node_take_object(result, obj);
+
+	/* Convert to string */
+	gchar *json_string = json_node_to_string(result, FALSE);
+
+	/* Freeing this node should also destroy the child nodes */
+	json_node_free(result);
+
+	g_printerr("&&&&&&&&&&&&&&&&&&\n");
+	g_printerr("%s\n", json_string);
+	g_printerr("&&&&&&&&&&&&&&&&&&\n");
+
+	gchar *reply = conboy_http_put(url, json_string, TRUE);
+
+	g_printerr("Reply from Snowy:\n");
+	g_printerr("%s\n", reply);
+
+	/* Parse answer and see if expected_rev fits or not */
+	JsonNoteList *note_list = json_get_note_list(reply);
+
+	if (note_list == NULL) {
+		g_set_error(error, 0, 0, "json_get_note_list() returned NULL");
+		return expected_rev - 1;
+	}
+
+	if (note_list->latest_sync_revision != expected_rev) {
+		g_printerr("WARN: Expected sync rev (%i) and actual sync rev (%i) are not the same\n", expected_rev, note_list->latest_sync_revision);
+		//g_set_error(error, 0, 0, "Expected sync rev (%i) and actual sync rev (%i) are not the same\n", expected_rev, note_list->latest_sync_revision);
+		return note_list->latest_sync_revision;
+	}
+
+	return expected_rev;
+}
+
+static void
+remove_deleted_notes_file()
+{
+	/* TODO: This filename and "synced_notes_txt" is plastered all over this file. Again and again... */
+	gchar *filename = g_strconcat(g_get_home_dir(), "/.conboy/deleted_notes.txt", NULL);
+	g_unlink(filename);
+	g_free(filename);
+}
+
 /* TODO: This function is way too long */
+/**
+ * 1.) Receive new/updated notes from server
+ * 2.) Process incoming deletion
+ * 3.) Send new/updated notes to server
+ * 4.) Send local deletion to server (Diff zwischen gesamter Server-Version und lokaler Version. Alles was lokal fehlt, kann auf dem Server geloescht werden)
+ */
 gpointer
 web_sync_do_sync (gpointer *user_data)
 {
@@ -283,7 +521,7 @@ web_sync_do_sync (gpointer *user_data)
 	GtkProgressBar *bar = data->bar;
 
 	/* Save and make uneditable */
-	/* FIXME: Function is returned often, witout making it editable again */
+	/* TODO: Function is returned often, witout making it editable again */
 	gdk_threads_enter();
 	gtk_text_view_set_editable(ui->view, FALSE);
 	note_save(ui);
@@ -324,6 +562,7 @@ web_sync_do_sync (gpointer *user_data)
 	g_printerr("Reply from /api/1.0/:: %s\n", reply);
 
 	gchar *api_ref = json_get_api_ref(reply);
+	g_free(reply);
 
 	g_printerr("Now asking: %s\n", api_ref);
 
@@ -333,18 +572,26 @@ web_sync_do_sync (gpointer *user_data)
 		gchar *msg = g_strconcat("Got no reply from: \n", api_ref, NULL);
 		web_sync_show_message(data, msg);
 		g_free(msg);
+		g_free(api_ref);
 		return;
 	}
 	web_sync_pulse_bar(bar);
+	g_free(api_ref);
 
 	g_printerr("Reply from /user/:: %s\n", reply);
 
 	/* Revision checks */
 	JsonUser *user = json_get_user(reply);
 	if (user == NULL) {
-		web_sync_show_message(data, "Could not parse server answer. Probably server error.");
+		if (g_strstr_len(reply, 30, "Subscription required")) {
+			web_sync_show_message(data, "Server said: 'Subscription required'. Please check that your local time is set correctly.\n");
+		} else {
+			web_sync_show_message(data, "Could not parse server answer. Probably server error.");
+		}
+		g_free(reply);
 		return;
 	}
+	g_free(reply);
 
 	if (user->latest_sync_revision < last_sync_rev) {
 		g_printerr("Server revision older than our revision\nU1 rev: %i   Local rev: %i\n", user->latest_sync_revision, last_sync_rev);
@@ -361,11 +608,23 @@ web_sync_do_sync (gpointer *user_data)
 	g_printerr("#   Api ref: %s\n", user->api_ref);
 	g_printerr("####################################\n");
 
+
+	/* Get notes from server */
 	gint added_note_count = 0;
 	gint changed_note_count = 0;
 	GList *local_changes = web_sync_incoming_changes(user, &last_sync_rev, last_sync_time, &added_note_count, &changed_note_count);
 	g_printerr("DEBUG: After web_sync_incoming_changes\n");
 
+
+	/*
+	 * Process deletions made on server
+	 * Notes that get deleted get also delted from local_changes, to
+	 * make sure we don't upload them again.
+	 */
+	gint deleted_note_count = 0;
+	local_changes = web_sync_delete_local_notes(user, local_changes, &deleted_note_count);
+
+
 	/*
 	 * Remaining local notes are newer on the client.
 	 * Send them to the server
@@ -375,21 +634,37 @@ web_sync_do_sync (gpointer *user_data)
 	GError *error = NULL;
 	int sync_rev = web_sync_send_notes(local_changes, user->api_ref, last_sync_rev + 1, last_sync_time, &uploaded_notes, &error);
 	web_sync_pulse_bar(bar);
+	g_list_free(local_changes);
 
+	gint deleted_on_server_count = 0;
+	if (!error) {
+		/*
+		 * Now that we updated notes on both sides, we can send local deletions
+		 * to the server.
+		 */
+		sync_rev = web_sync_send_local_deletions(user->api_ref, sync_rev + 1, &deleted_on_server_count, &error);
+	}
 
+	json_user_free(user);
 
 	gchar msg[1000];
 
 	if (!error) {
+		/* Store new sync_rev and sync_time */
 		settings_save_last_sync_revision(sync_rev);
 		settings_save_last_sync_time(time(NULL));
 
-		g_sprintf(msg, "<b>%s</b>\n\n%s: %i\n%s: %i\n%s: %i\n%s: %i",
+		/* Update synced_notes and deleted_notes file */
+		update_synced_notes_file();
+		remove_deleted_notes_file();
+
+		g_sprintf(msg, "<b>%s</b>\n\n%s: %i\n%s: %i\n%s: %i\n%s: %i\n%s: %i",
 				"Synchonization completed",
 				"Added notes", added_note_count,
 				"Changed notes", changed_note_count,
-				"Deleted notes", 0,
-				"Uploaded notes", uploaded_notes);
+				"Deleted notes", deleted_note_count,
+				"Uploaded notes", uploaded_notes,
+				"Deleted on server", deleted_on_server_count);
 
 	} else {
 		g_printerr("ERROR: %s\n", error->message);
@@ -400,9 +675,6 @@ web_sync_do_sync (gpointer *user_data)
 	/*
 	 * Sync finished
 	 */
-	g_free(api_ref);
-	g_list_free(local_changes);
-	/* TODO: Free json stuff */
 
 	/* Show message to user */
 	web_sync_show_message(data, msg);
@@ -415,13 +687,7 @@ web_sync_do_sync (gpointer *user_data)
 
 	/* If not possible, because it was deleted or there was no previous note, get latest */
 	if (note == NULL) {
-		note = conboy_note_store_get_latest(app_data->note_store);
-	}
-
-	if (note == NULL) {
-		/* TODO: Show demo note */
-		note = conboy_note_new();
-		g_printerr("ERROR: No notes to display\n");
+		note = conboy_note_store_get_latest_or_new(app_data->note_store);
 	}
 
 	/* Load again and make editable */
@@ -670,6 +936,7 @@ web_sync_authenticate(gchar *url, GtkWindow *parent)
 	g_free(request);
 
 	JsonApi *api = json_get_api(reply);
+	g_free(reply);
 
 	/*g_printerr("###################################\n");
 	g_printerr("Request token url: %s\n", api->request_token_url);
@@ -679,6 +946,7 @@ web_sync_authenticate(gchar *url, GtkWindow *parent)
 
 	/* Get auth link url */
 	gchar *link = conboy_get_request_token_and_auth_link(api->request_token_url, api->authorize_url);
+	json_api_free(api);
 
 	if (link == NULL) {
 		ui_helper_show_confirmation_dialog(parent, "Could not connect to host.", FALSE);
@@ -691,6 +959,7 @@ web_sync_authenticate(gchar *url, GtkWindow *parent)
 			OSSO_BROWSER_OPEN_NEW_WINDOW_REQ, NULL,
 			DBUS_TYPE_STRING, link, DBUS_TYPE_INVALID);
 	g_printerr("Opening browser with URL: >%s<\n", link);
+	g_free(link);
 
 	GtkWidget *dialog = ui_helper_create_cancel_dialog(parent, "Please grant access on the website of your service provider that just opened.\nAfter that you will be automatically redirected back to Conboy.");
 
@@ -740,7 +1009,6 @@ web_sync_authenticate(gchar *url, GtkWindow *parent)
 		return FALSE;
 	}
 
-
 	if (result == 666) {
 		kill_callback_thread();
 		ui_helper_show_confirmation_dialog(parent, "You have manually canceled the authentication process.", FALSE);
