@@ -25,7 +25,12 @@
 
 #include "app_data.h"
 #include "metadata.h"
-#include "note_list_store.h"
+#include "conboy_note_store.h"
+
+#ifndef GLIB_HAS_PCRE
+#include "gregex.h"
+#endif
+
 #include "note_linker.h"
 
 #ifndef max
@@ -35,28 +40,9 @@
 typedef struct {
 	gint   start_offset;
 	gint   end_offset;
-	Note  *note;
+	ConboyNote  *note;
 } SearchHit;
 
-/* TODO: Optimize this. We need a place where all titles are stored, info which is the longest, ... */
-static gint
-get_length_of_longest_title()
-{
-	gint max_length = 0;
-	AppData *app_data = app_data_get();
-	GtkTreeIter iter;
-	GtkTreeModel *model = GTK_TREE_MODEL(app_data->note_store);
-	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
-
-	while (valid) {
-		Note *note;
-		gtk_tree_model_get(model, &iter, NOTE_COLUMN, &note, -1);
-		max_length = max(g_utf8_strlen(note->title, -1), max_length);
-		valid = gtk_tree_model_iter_next(model, &iter); /* TODO: Too much casting in this function. Maybe introduce note_list_store_iter_next() etc... */
-	}
-
-	return max_length;
-}
 
 static
 GSList* find_titles(gchar *haystack) {
@@ -78,7 +64,7 @@ GSList* find_titles(gchar *haystack) {
 	gboolean valid = gtk_tree_model_get_iter_first(model, &iter);
 	while (valid) {
 		gchar *u_needle;
-		Note *note;
+		ConboyNote *note;
 		gtk_tree_model_get(model, &iter, NOTE_COLUMN, &note, -1);
 		
 		/* If title is empty, look at next note. An empty title would bring us into an infinit loop */
@@ -143,7 +129,7 @@ extend_block(GtkTextIter *start_iter, GtkTextIter *end_iter, gint max_len, GtkTe
 }
 
 static void
-highlight_titles(Note *note, GtkTextBuffer *buffer, GtkTextIter *start_iter, GtkTextIter *end_iter)
+highlight_titles(ConboyNote *note, GtkTextBuffer *buffer, GtkTextIter *start_iter, GtkTextIter *end_iter)
 {
 	GtkTextTag *url_tag  = gtk_text_tag_table_lookup(buffer->tag_table, "link:url");
 	/* For all titles look if they occure in haystack. For all matches apply tag */
@@ -188,20 +174,93 @@ highlight_titles(Note *note, GtkTextBuffer *buffer, GtkTextIter *start_iter, Gtk
 	g_slist_free(hits);
 }
 
-void auto_highlight_links(Note *note, GtkTextIter *start_iter, GtkTextIter *end_iter)
+void
+auto_highlight_links(UserInterface *ui, GtkTextIter *start_iter, GtkTextIter *end_iter)
 {
-	GtkTextBuffer *buffer = note->ui->buffer;
+	/* Copy text iters because extend_block will change them */
+	GtkTextIter start = *start_iter;
+	GtkTextIter end = *end_iter;
+	
+	GtkTextBuffer *buffer = ui->buffer;
 	GtkTextTag *link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
-	int max_len = get_length_of_longest_title();
+	AppData *app_data = app_data_get();
 	
 	/* Grow the block which will be checked for links */
-	extend_block(start_iter, end_iter, max_len, link_tag);
+	extend_block(&start, &end, app_data->note_store->max_title_length, link_tag);
 	
 	/* Remove existing link tag */
-	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
+	gtk_text_buffer_remove_tag(buffer, link_tag, &start, &end);
 
 	/* Add links */
-	highlight_titles(note, buffer, start_iter, end_iter);
+	highlight_titles(ui->note, buffer, &start, &end);
+}
+
+
+#define REGEX "((\\b((news|http|https|ftp|file|irc)://|mailto:|(www|ftp)\\.|\\S*@\\S*\\.)|(?<=^|\\s)/\\S+/|(?<=^|\\s)~/\\S+)\\S*\\b/?)"
+GRegex *_regex = NULL;
+
+void
+auto_highlight_urls(UserInterface *ui, GtkTextIter *start_iter, GtkTextIter *end_iter)
+{
+	/* Copy text iters because extend_block will change them */
+	GtkTextIter start = *start_iter;
+	GtkTextIter end = *end_iter;
+	
+	GtkTextBuffer *buffer = ui->buffer;
+	GtkTextTag *link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:url");
+
+	/* Grow the block by 256 chars (max url length) which will be checked for links */
+	extend_block(&start, &end, 256, link_tag);
+	
+	/* Remove existing link tag */
+	gtk_text_buffer_remove_tag(buffer, link_tag, &start, &end);
+
+	/* The piece of text we are interested in */
+	gchar *str = gtk_text_iter_get_slice(&start, &end);
+	
+	/* Only compile the regex once */
+	if (_regex == NULL) {
+		_regex = g_regex_new(REGEX, G_REGEX_CASELESS, 0, NULL);
+	}
+	
+	/* Match the regex */
+	GMatchInfo *match_info;
+	g_regex_match(_regex, str, 0, &match_info);
+	
+	/* Iterate through the matches and apply the link tag */
+	while (g_match_info_matches(match_info))
+	{
+		gint start_pos, end_pos;
+		
+		/* Get start and end position of the match.
+		 * 
+		 * The Problem is g_match_info_fetch_pos() get the position
+		 * in bytes. Which breaks things when we use characters that
+		 * are bigger than 1 byte (e.g. umlauts, bullets, etc).
+		 * 
+		 * So we need to recalculate the position.
+		 */
+		g_match_info_fetch_pos(match_info, 0, &start_pos, &end_pos);
+		
+		gchar *start_ptr = str + start_pos;
+		gchar *end_ptr   = str + end_pos;
+		
+		start_pos = g_utf8_pointer_to_offset(str, start_ptr);
+		end_pos   = g_utf8_pointer_to_offset(str, end_ptr);
+		
+		/* Move the iters and apply tag */
+		GtkTextIter xstart = start;
+		gtk_text_iter_forward_chars(&xstart, start_pos);
+		
+		GtkTextIter xend = xstart;
+		gtk_text_iter_forward_chars(&xend, end_pos - start_pos);
+		
+		gtk_text_buffer_apply_tag(buffer, link_tag, &xstart, &xend);
+		
+		g_match_info_next(match_info, NULL);
+	}
+	
+	g_match_info_free(match_info);
 }
 
 
